from chart import *
from parser import *
from dtparser import *
from emailparser import *

from rdflib import *

"""
Semantic Web Pidgin: Parser driver agent
Max Goldman and Robin Stewart
"""

class Agent:
    """
    Drives parsers.
    """
    def __init__(self):
        # use our own information store for now
        self._store = ConjunctiveGraph()
        #self._store.load("http://www.w3.org/1999/02/22-rdf-syntax-ns")
        self._parsers = [ ]

    def add_ground_parser(self, parser):
        """
        Add the specified parser.
        """
        self._parsers.append(parser)

    def add_sw_parser(self, model, entities=None):
        """
        Add parsers for the specified semantic web model and entities.
        """
        self._parsers.append(SemanticWebParser(self._store, model, entities))

    def interpret(self, input):
        """
        Attempt to interpret the input, which should be a list of strings,
        such as ["david", "karger", "tomorrow", "6pm", "stata"].
        """
        tokens = [ Token(RDF_STRING, t.lower(), 1.0) for t in input.split() ]
        if len(tokens) < 1:
            return None
        chart = Chart(tokens)
        ### print "tokens", tokens

        attempted_token_lists = [ ]
        new_parses = True
        while new_parses:
            new_parses = False
            for length in range(1, len(chart)+1):
                for start in range(0, len(chart)-length+1):
                    end = start + length - 1
                    for token_list in chart.possible_token_lists(start, end):
                        weight_list = [ token.weight() for token in token_list ]
                        if token_list in attempted_token_lists:
                            continue
                        attempted_token_lists.append(token_list)
                        prior_weight = reduce(lambda x,y: x*y, weight_list)
                        ### print "try", token_list
                        for parser in self._parsers:
                            parses = parser.parse(token_list)
                            [ parse.reweight(prior_weight) for parse in parses ]
                            if chart.add_parses(start, end, parses):
                                new_parses = True

        parses = chart.parses(0, len(tokens)-1)
        parses = filter(lambda t: t not in tokens, parses)
        if len(parses) < 1:
            return None

        token = reduce(lambda a,b: (a if a.weight() > b.weight() else b), parses)
        return token

    def print_token(self, token):
        """
        Pretty-print the given token, including, weight, type, and object
        contents.
        """
        print token.weight(), token.type()
        object = token.object()
        if object.__class__ == URIRef:
            print "Entity"
            print " ", object
        elif object.__class__ == dict:
            print "Dictionary"
            for (key, value) in object.iteritems():
                print " ", key + ":"
                print "     ", value
        else:
            print object.__class__
            print " ", object
