from __future__ import division  # normal floating point division

from parser import *

import parsedatetime.parsedatetime as pdt 
import parsedatetime.parsedatetime_consts as pdc

"""
Semantic Web Pidgin: Date and time parser
Max Goldman and Robin Stewart
"""

class DateTimeParser (Parser):

    def __init__(self):
	"""Creates a date-time parser"""
	self.dtparser = pdt.Calendar(pdc.Constants())

    def parse(self, tokens):
	"""The main parse method """
	# the parser doesn't like text numbers like "two"
	# but we'll ignore that for now
	
	count = 0
	for tok in tokens:
	    # reject if anything is not a string token:
	    count += 1
	    if tok.type() != RDF_STRING:
		return []
	
	# if first token is not parseable, reject:
	(date,flag) = self.dtparser.parse(tokens[0].object())
	if flag == 0:
	    return []
	# if last token is not parseable, reject:
	(date,flag) = self.dtparser.parse(tokens[count - 1].object())
	if flag == 0:
	    return []
	
	    
	# construct the full string:
	phrase = " ".join([tok.object() for tok in tokens])

	# now see if this string can be parsed:
	(date,flag) = self.dtparser.parse(phrase)
	    ### flag meanings: ###
	    # 0 = not parsed at all
	    # 1 = parsed as a C{date}
	    # 2 = parsed as a C{time}
	    # 3 = parsed as a C{datetime}
	if flag == 0:
	    return []
	else:
	    return [Token(RDF_DATETIME, date, 1/count)]
	    #return [Token(RDF_DATETIME, date, .001)]

	    
    def match_preposition(self, token):
	"""Returns the token if it is a preposition match. """
	if token.type() == RDF_STRING:
	    for prep in self.prepositions:
		if prep == token.object:
		    return token
	# if got this far
	return None

def main():

    dtconst = pdc.Constants()
    dtparser = pdt.Calendar(dtconst)

    str_tokens = ["meet", "with", "karger", "at", "5pm", "on", "2/3/08"]

    # the parser doesn't like text numbers like "two"
    # but we'll ignore that for now

    date_tokens = []
    for tok in str_tokens:
	(date,flag) = dtparser.parse(tok)
	# flag meanings:
	# 0 = not parsed at all
	# 1 = parsed as a C{date}
	# 2 = parsed as a C{time}
	# 3 = parsed as a C{datetime}
	if flag > 0:
	    date_tokens.append(date)
	else:
	    # can't parse this string of tokens
	    continue
	    
    # now see how much we can consume
    print date_tokens


if __name__ == '__main__':
    main()

