from __future__ import division  # normal floating point division

from parser import *
import re

"""
Semantic Web Pidgin: Email parser
Max Goldman and Robin Stewart
"""

class EmailParser (Parser):

    def __init__(self):
	"""Creates an email parser"""
	# "strict" RE only accepts complete email addresses
	self.strictRE = re.compile('[a-zA-Z0-9_.-]+@[a-zA-Z0-9_.-]+\.[a-z]{3}')
	# "shorthand" RE accepts things like "stewart@csail"
	self.shortRE = re.compile('[a-zA-Z0-9_.-]+@[a-zA-Z0-9_.-]+')

    def parse(self, tokens):
	"""The main parse method """

	# this parser only accepts a single string token
	if len(tokens) > 1 or tokens[0].type() != RDF_STRING:
	    return []
	
	# now see if the string matches the email regular expression
	if self.strictRE.match(tokens[0].object()):
	    return [Token(RDF_EMAIL, tokens[0].object(), 1)]
	elif self.shortRE.match(tokens[0].object()):
	    return [Token(RDF_EMAIL, tokens[0].object(), 0.8)]
	else:
	    return []


# testing code:
def main():
    token = Token(RDF_STRING, "stewart.csail.mit.edu", 1)
    ep = EmailParser()
    print ep.parse([token])


if __name__ == '__main__':
    main()

